@@ -259,7 +259,7 @@ class Application(QApplication, Singleton):
 
         self.systconf['dynamips'] = systemDynamipsConf()
         confo = self.systconf['dynamips']
-        confo.path = ConfDB().get('Dynamips/hypervisor_path', unicode(''))
+        confo.path = ConfDB().get('Dynamips/hypervisor_path', unicode('/usr/local/bin/dynamips'))
         confo.port = int(ConfDB().get('Dynamips/hypervisor_port', 7200))
         confo.baseUDP = int(ConfDB().get('Dynamips/hypervisor_baseUDP', 10000))
         confo.baseConsole = int(ConfDB().get('Dynamips/hypervisor_baseConsole', 2000))
