@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
+# Copyright (C) 2007-2008 GNS3 Dev Team
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
@@ -13,11 +15,13 @@
 # Contact: contact@gns3.net
 #
 
-import os, sys, cmd, time
+import os, sys, cmd, time, shutil
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynagen as Dynagen_Namespace
 import GNS3.Dynagen.dynamips_lib as lib
+import GNS3.NETFile as netfile
 from PyQt4 import QtCore, QtGui
+from GNS3.Utils import translate
 from GNS3.Dynagen.console import Console as Dynagen_Console, getItems
 from GNS3.External.PyCutExt import PyCutExt
 from GNS3.Node.IOSRouter import IOSRouter
@@ -30,18 +34,18 @@ class Console(PyCutExt, Dynagen_Console):
                 "exit", "help", "import", "push", "resume",
                 "shell", "stop", "ver", "confreg",
                 "export", "hist", "list", "py",
-                "save", "show", "suspend"])
+                "save", "show", "suspend", "hypervisors"])
 
     def __init__(self, parent):
         """ Initialise the Console widget
         """
+
         # Set the prompt, for Dynagen.Console and PyCutExt
         self.prompt = '=> '
         sys.ps1 = '=> '
-        sys.ps2 = '.. '
 
         # Set introduction message
-        self.intro = 'Dynagen management console for Dynamips (adapted for GNS3)\nCopyright (c) 2005-2007 Greg Anuzelli\nCopyright (c) 2007 GNS3 Project'
+        self.intro = 'Dynagen management console for Dynamips (adapted for GNS3)\nCopyright (c) 2008 GNS3 Project'
 
         # Parent class initialisation
         try:
@@ -50,7 +54,7 @@ class Console(PyCutExt, Dynagen_Console):
             self.colorizer.keywords = self.keywords
             self._Dynagen_Console_init()
         except Exception,e:
-            sys.stderr.write(e.message) 
+            sys.stderr.write(e.message)
 
     def _Dynagen_Console_init(self):
         """ Dynagen Console class initialisation
@@ -58,8 +62,8 @@ class Console(PyCutExt, Dynagen_Console):
         """
 
         cmd.Cmd.__init__(self)
-        self.namespace = Dynagen_Namespace 
-        debuglevel = self.namespace.debuglevel
+        self.namespace = Dynagen_Namespace
+        self.dynagen = globals.GApp.dynagen
 
     def onKeyPress_Tab(self):
         """ Imitate cmd.Cmd.complete(self, text, state) function
@@ -80,7 +84,7 @@ class Console(PyCutExt, Dynagen_Console):
                     compfunc = self.completenames
         else:
             compfunc = self.completenames
-        
+
         self.completion_matches = compfunc(cmd, line, 0, 0)
         if self.completion_matches is not None:
             # Eliminate repeating values
@@ -98,7 +102,7 @@ class Console(PyCutExt, Dynagen_Console):
             if len(self.completion_matches) == 1:
                 newLine = self.completion_matches[0] + " " + args
                 self.line = QtCore.QString(newLine)
-                self.point = len(newLine) 
+                self.point = len(newLine)
             # Else, display possible values
             else:
                 self.write("\n")
@@ -122,15 +126,17 @@ class Console(PyCutExt, Dynagen_Console):
             self.more = self.onecmd(source)
         except Exception,e:
             print e
-            globals.GApp.workspace.switchToMode_Design()
 
-        if self.more:
-            self.write(sys.ps2)
-        else:
-            self.write(sys.ps1)
-            self.lines = []
-            self._clearLine()
-            
+        self.write(self.prompt)
+        self.lines = []
+        self._clearLine()
+
+    def do_hypervisors(self, args):
+        """hypervisors \nshow the hypervisors started by the hypervisor manager"""
+
+        if globals.GApp.HypervisorManager:
+            globals.GApp.HypervisorManager.showHypervisors()
+
     def do_start(self, args):
         """start  {/all | router1 [router2] ...}\nstart all or a specific router(s)"""
 
@@ -138,12 +144,16 @@ class Console(PyCutExt, Dynagen_Console):
             Dynagen_Console.do_start(self, args)
             devices = args.split(' ')
             for node in globals.GApp.topology.nodes.values():
-                if type(node) == IOSRouter and (node.hostname in devices or '/all' in devices):
+                if isinstance(node, IOSRouter) and (node.hostname in devices or '/all' in devices):
                     node.startupInterfaces()
                     globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(node.hostname, 'running')
-        except:
-            globals.GApp.workspace.switchToMode_Design()
-        
+        except lib.DynamipsError, msg:
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"),  unicode(msg))
+        except lib.DynamipsWarning,  msg:
+            QtGui.QMessageBox.warning(self,  node.hostname + ': ' + translate("Console", "Dynamips warning"),  unicode(msg))
+        except (lib.DynamipsErrorHandled,  socket.error):
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"), translate("Console", "Connection lost"))
+
     def do_stop(self, args):
         """stop  {/all | router1 [router2] ...}\nstop all or a specific router(s)"""
 
@@ -151,11 +161,15 @@ class Console(PyCutExt, Dynagen_Console):
             Dynagen_Console.do_stop(self, args)
             devices = args.split(' ')
             for node in globals.GApp.topology.nodes.values():
-                if type(node) == IOSRouter and (node.hostname in devices or '/all' in devices):
+                if isinstance(node, IOSRouter) and (node.hostname in devices or '/all' in devices):
                     node.shutdownInterfaces()
                     globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(node.hostname, 'stopped')
-        except:
-            globals.GApp.workspace.switchToMode_Design()
+        except lib.DynamipsError, msg:
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"),  unicode(msg))
+        except lib.DynamipsWarning,  msg:
+            QtGui.QMessageBox.warning(self,  node.hostname + ': ' + translate("Console", "Dynamips warning"),  unicode(msg))
+        except (lib.DynamipsErrorHandled,  socket.error):
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"), translate("Console", "Connection lost"))
 
     def do_suspend(self, args):
         """suspend  {/all | router1 [router2] ...}\nsuspend all or a specific router(s)"""
@@ -164,11 +178,15 @@ class Console(PyCutExt, Dynagen_Console):
             Dynagen_Console.do_suspend(self, args)
             devices = args.split(' ')
             for node in globals.GApp.topology.nodes.values():
-                if type(node) == IOSRouter and (node.hostname in devices or '/all' in devices):
+                if isinstance(node, IOSRouter) and (node.hostname in devices or '/all' in devices):
                     node.suspendInterfaces()
                     globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(node.hostname, 'suspended')
-        except:
-            globals.GApp.workspace.switchToMode_Design()
+        except lib.DynamipsError, msg:
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"),  unicode(msg))
+        except lib.DynamipsWarning,  msg:
+            QtGui.QMessageBox.warning(self,  node.hostname + ': ' + translate("Console", "Dynamips warning"),  unicode(msg))
+        except (lib.DynamipsErrorHandled,  socket.error):
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"), translate("Console", "Connection lost"))
 
     def do_resume(self, args):
         """resume  {/all | router1 [router2] ...}\nresume all or a specific router(s)"""
@@ -177,39 +195,43 @@ class Console(PyCutExt, Dynagen_Console):
             Dynagen_Console.do_resume(self, args)
             devices = args.split(' ')
             for node in globals.GApp.topology.nodes.values():
-                if type(node) == IOSRouter and (node.hostname in devices or '/all' in devices):
+                if isinstance(node, IOSRouter) and (node.hostname in devices or '/all' in devices):
                     node.startupInterfaces()
                     globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(node.hostname, 'running')
-        except:
-            globals.GApp.workspace.switchToMode_Design()
-            
+        except lib.DynamipsError, msg:
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"),  unicode(msg))
+        except lib.DynamipsWarning,  msg:
+            QtGui.QMessageBox.warning(self,  node.hostname + ': ' + translate("Console", "Dynamips warning"),  unicode(msg))
+        except (lib.DynamipsErrorHandled,  socket.error):
+            QtGui.QMessageBox.critical(self, node.hostname + ': ' + translate("Console", "Dynamips error"), translate("Console", "Connection lost"))
+
     def do_reload(self, args):
         """reload  {/all | router1 [router2] ...}\nreload all or a specific router(s)"""
 
         self.do_stop(args)
         self.do_start(args)
-            
+
     def do_exit(self,  args):
-        """ Returns to design mode """
-        
-        globals.GApp.workspace.switchToMode_Design()
-        
+        """clear the topology"""
+
+        globals.GApp.topology.clear()
+
     def do_disconnect(self,  args):
-        """ Returns to design mode """
-        
-        globals.GApp.workspace.switchToMode_Design()
-        
+        """clear the topology"""
+
+        globals.GApp.topology.clear()
+
     def do_py(self,  args):
-        """ Not implemented in GNS3 """
-        
-        print 'Sorry, not implemented in GNS3'
-    
+        """not implemented in GNS3"""
+
+        print translate("Console", "Sorry, not implemented in GNS3")
+
     def do_hist(self, args):
-        """Print a list of commands that have been entered"""
+        """print a list of commands that have been entered"""
 
         for entry in self.history:
             print unicode(entry)
-        
+
     def do_idlepc(self, args):
         """idlepc {get|set|show|save|idlemax|idlesleep|showdrift} device [value]
 idlepc save device [default]
@@ -243,40 +265,23 @@ Examples:
             if len(params) < 1:
                 print Dynagen_Console.do_idlepc.__doc__
                 return
-                
+
             if command == 'save':
-                print 'Sorry, save has not been implemented in GNS3 yet'
+                print translate("Console", "Sorry, not implemented in GNS3")
                 return
-            
+
             if command == 'get' or command == 'show':
                 device = params[0]
                 if command == 'get':
-                    if self.namespace.devices[device].idlepc != None:
-                        print '%s already has an idlepc value applied.' % device
+                    if self.dynagen.devices[device].idlepc != None:
+                        print unicode(translate("Console", "%s already has an idlepc value applied.")) % device
                         return
-                    print 'Please wait while gathering statistics...'
+
+                    print translate("Console", "Please wait while gathering statistics...")
                     globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
-                    result = self.namespace.devices[device].idleprop(lib.IDLEPROPGET)
-                    
-#                    progress = QtGui.QProgressDialog('IDLEPC', 'Abort', 0, 20, self)
-#                    progress.setMinimum(1)
-#                    progress.setWindowModality(QtCore.Qt.WindowModal)
-#                    self.processEvents(QtCore.QEventLoop.AllEvents)
-#                    for nb in range(1, 21):
-#                        print nb
-#                        progress.setValue(nb)
-#                        time.sleep(1)
-#                        self.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
-#                        if progress.wasCanceled():
-#                            progress.reset()
-#                            break
-#                    progress.setValue(20)
-#                    progress.deleteLater()
-#                    progress = None
-
-
+                    result = self.dynagen.devices[device].idleprop(lib.IDLEPROPGET)
                 elif command == 'show':
-                    result = self.namespace.devices[device].idleprop(lib.IDLEPROPSHOW)
+                    result = self.dynagen.devices[device].idleprop(lib.IDLEPROPSHOW)
                 result.pop()        # Remove the '100-OK' line
                 idles = {}
                 i = 1
@@ -297,39 +302,40 @@ Examples:
 
                 # Allow user to choose a value by number
                 if len(idles) == 0:
-                    print 'No idlepc values found\n'
+                    print translate("Console", "No idlepc values found")
                 else:
-                    output = "Potentially better idlepc values marked with '*'\nEnter the number of the idlepc value to apply [1-%i] or ENTER for no change:\n" % len(idles) + output
+                    output = unicode(translate("Console", "Potentially better idlepc values marked with '*'\nEnter the number of the idlepc value to apply [1-%i] or ENTER for no change:\n")) % len(idles) + output
                     globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
                     (selection,  ok) = QtGui.QInputDialog.getText(globals.GApp.mainWindow, 'idlepc',
                                           output, QtGui.QLineEdit.Normal)
-                    
+
                     if not ok:
-                        print 'No changes made'
+                        print translate("Console", "No changes made")
                         return
                     selection = str(selection)
                     if selection == "":
-                        print 'No changes made'
+                        print translate("Console", "No changes made")
                         return
-                        
+
                     try:
-                        self.namespace.devices[device].idleprop(lib.IDLEPROPSET, idles[int(selection)])
-                        print "Applied idlepc value %s to %s\n" % (idles[int(selection)], device)
+                        self.dynagen.devices[device].idleprop(lib.IDLEPROPSET, idles[int(selection)])
+                        print unicode(translate("Console", "Applied idlepc value %s to %s\n")) % (idles[int(selection)], device)
                         for node in globals.GApp.topology.nodes.values():
-                            if type(node) == IOSRouter and node.hostname == device:
-                                if node.config.image and globals.GApp.iosimages.has_key(node.config.image):
-                                    image = globals.GApp.iosimages[node.config.image]
+                            if isinstance(node, IOSRouter) and node.hostname == device:
+                                router = node.get_dynagen_device()
+                                if globals.GApp.iosimages.has_key(router.dynamips.host + ':' + router.image):
+                                    image = globals.GApp.iosimages[router.dynamips.host + ':' + router.image]
                                     image.idlepc =  idles[int(selection)]
                     except:
-                        print "Can't apply idlepc value"
+                        print translate("Console", "Can't apply idlepc value")
             else:
                 Dynagen_Console.do_idlepc(self, args)
-            
+
         except ValueError:
-            print '***Error: Incorrect number of paramaters or invalid parameters'
+            print translate("Console", "Incorrect number of paramaters or invalid parameters")
             return
         except KeyError:
-            print '***Error: Unknown device: ' + device
+            print unicode(translate("Console", "Unknown device: %s")) % device
             return
         except lib.DynamipsError, e:
             print e
@@ -337,39 +343,36 @@ Examples:
 
     def do_save(self, args):
         """save {/all | router1 [router2] ...}\nstores router configs in the network file"""
-    
+
         if not globals.GApp.workspace.projectFile:
-            print 'You have to save your topology before using save'
+            print translate("Console", "You have to save your topology before using save")
         else:
             Dynagen_Console.do_save(self, args)
-    
+
     def do_push(self, args):
         """push {/all | router1 [router2] ...}\npushes router configs from the network file to the router's nvram"""
-    
+
         if not globals.GApp.workspace.projectFile:
-            print 'You have to save your topology before using push'
+            print translate("Console", "You have to save your topology before using push")
         else:
             Dynagen_Console.do_push(self, args)
 
     def do_telnet(self, args):
         """telnet  {/all | router1 [router2] ...}\nconnect to the console(s) of all or a specific router(s)\nThis is identical to the console command."""
-        
+
         self.do_console(args)
-        
+
     def do_console(self, args):
         """console  {/all | router1 [router2] ...}\nconnect to the console(s) of all or a specific router(s)\n"""
-        
+
         devices = args.split(' ')
         for node in globals.GApp.topology.nodes.values():
-            if type(node) == IOSRouter and (node.hostname in devices or '/all' in devices):
+            if isinstance(node, IOSRouter) and (node.hostname in devices or '/all' in devices):
                 node.console()
 
     def do_export(self, args):
         """export {/all | router1 [router2] ...} \"directory\"\nsaves router configs individual files in \"directory\"\nEnclose the directory in quotes if there are spaces in the filespec."""
-        
-        if not globals.GApp.workspace.projectFile:
-            print 'You have to save your topology before using export'
-            return
+
         if '?' in args or args.strip() == '':
             print Dynagen_Console.do_export.__doc__
             return
@@ -384,33 +387,55 @@ Examples:
             return
 
          # The last item is the directory (or should be anyway)
-        directory = items.pop()
-        try:
-            netdir = os.getcwd()
-            subdir = os.path.dirname(self.namespace.FILENAME)
-            if subdir != '':
-                os.chdir(subdir)
-            if os.path.isdir(directory):
-                # Directory exists
-                print directory
-                (result,  ok) = QtGui.QInputDialog.getText(globals.GApp.mainWindow, 'Destination directory',
-                  'The directory already exists. Ok to overwrite (Y/N)?', QtGui.QLineEdit.Normal)
-                if not ok:
-                    return
-                if str(result).lower() != 'y':
-                    os.chdir(netdir)        # Reset the current working directory
-                else:
-                    os.rmdir(directory)
-        except OSError:
-            return
+        directory = items.pop().strip('"')
 
-        Dynagen_Console.do_export(self, args)
+        if not os.access(directory, os.F_OK):
+            try:
+                os.mkdir(directory)
+            except (OSError, IOError), e:
+                print translate("Console",  "Cannot create %s: %s") % (directory, e.strerror)
+                return
+
+        if '/all' in items:
+            # Set devices to all the devices
+            devices = self.dynagen.devices.values()
+        else:
+            devices = []
+            for device in items:
+                # Create a list of all the device objects
+                try:
+                    devices.append(self.dynagen.devices[device])
+                except KeyError:
+                    print 'unknown device: ' + device
+                    return
+
+        save = globals.GApp.workspace.projectConfigs
+        globals.GApp.workspace.projectConfigs = directory
+        net = netfile.NETFile()
+        for device in devices:
+            if isinstance(device, lib.Router):
+                net.export_router_config(device)
+        globals.GApp.workspace.projectConfigs = save
 
     def do_import(self, args):
         """import {/all | router1 [router2] \"directory\"\nimport all or individual configuration files \nEnclose the directory or filename in quotes if there are spaces in the filespec."""
-        
-        if not globals.GApp.workspace.projectFile:
-            print 'You have to save your topology before using import'
+
+        Dynagen_Console.do_import(self, args)
+
+    def do_debug(self, args):
+        """debug [level]\nActivate/Desactivate debugs\nLevel 0: no debugs\nLevel 1: dynamips lib debugs only\nLevel 2: GNS3 debugs only\nLevel 3: GNS3 debugs and dynamips lib debugs"""
+    
+        if len(args) == 1:
+            try:
+                level = int(args[0])
+                if level == 1 or level == 3:
+                    globals.debugLevel = level
+                    lib.setdebug(True)
+                if level == 0 or level == 2:
+                    globals.debugLevel = level
+                    lib.setdebug(False)
+            except:
+                print self.do_debug.__doc__
         else:
-            Dynagen_Console.do_import(self, args)
+            print self.do_debug.__doc__
     
