@@ -189,10 +189,10 @@ class NETFile(object):
         if device.model == 'c3725' or device.model == 'c3745':
             conf_image.platform = 'c3700'
         else:
-            conf_image.platform = device.model
-        conf_image.chassis = device.model_string
+            conf_image.platform = str(device.model)
+        conf_image.chassis = str(device.model_string)
         if device.idlepc:
-            conf_image.idlepc = device.idlepc
+            conf_image.idlepc = str(device.idlepc)
         conf_image.default_ram = device.ram
         conf_image.hypervisor_port = device.dynamips.port
         conf_image.default = False
@@ -413,7 +413,7 @@ class NETFile(object):
                     hostname = unicode(hostname)
                     symbol = unicode(gns3data[section]['symbol'])
                     if not globals.GApp.scene.renders.has_key(symbol):
-                        print unicode(translate("NETFile", "%s: cannot find %s symbol")) % (hostname, symbol)
+                        print unicode(translate("NETFile", "%s: cannot find %s symbol, please check this symbol is in your node list and reload the .net file")) % (hostname, symbol)
                         continue
                     renders = globals.GApp.scene.renders[symbol]
                     decorative_node = DecorativeNode(renders['normal'], renders['selected'])
@@ -834,7 +834,7 @@ class NETFile(object):
                 if connections:
                     config['connections'] = connections.strip()
             # record notes
-            elif isinstance(item, Annotation):
+            elif isinstance(item, Annotation) and item.autoGenerated == False:
                 if not self.dynagen.running_config.has_key('GNS3-DATA'):
                     self.dynagen.running_config['GNS3-DATA'] = {}
                 self.dynagen.running_config['GNS3-DATA']['NOTE ' + str(note_nb)] = {}
