@@ -44,6 +44,13 @@ class ATMSW(AbstractNode):
 
         # assign a new hostname
         global atmsw_id
+        
+        # check if hostname has already been assigned
+        for node in globals.GApp.topology.nodes.itervalues():
+            if 'ATM' + str(atmsw_id) == node.hostname:
+                atmsw_id = atmsw_id + 1
+                break
+        
         self.hostname = 'ATM' + str(atmsw_id)
         atmsw_id = atmsw_id + 1
         AbstractNode.setCustomToolTip(self)
