@@ -19,6 +19,7 @@
 # Contact: contact@gns3.net
 #
 
+import re
 import GNS3.Globals as globals
 from PyQt4 import QtCore, QtGui, QtSvg
 from GNS3.Utils import translate, debug
@@ -56,7 +57,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         # scene settings
         self.setFlags(self.ItemIsMovable | self.ItemIsSelectable | self.ItemIsFocusable)
         self.setAcceptsHoverEvents(True)
-        self.setZValue(1)
+        self.setZValue(2)
         self.setSharedRenderer(self.__render_normal)
         
         # x&y position for hostname
@@ -89,12 +90,15 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
                                           translate("AbstractNode", "Hostname:"), QtGui.QLineEdit.Normal,
                                           self.hostname)
         if ok and text:
-            text = unicode(text).replace(' ', '')
-            text = text.replace(':', '.')
+            text = unicode(text)
+            if not re.search(r"""^[\w,.]*$""", text, re.UNICODE):
+                QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hostname"), 
+                                           translate("AbstractNode", "Please use only alphanumeric characters"))
+                self.changeHostname()
+                return
             for node in globals.GApp.topology.nodes.itervalues():
                 if text == node.hostname:
                     QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hostname"),  translate("AbstractNode", "Hostname already used"))
-                    return
             self.reconfigNode(text)
             if self.__flag_hostname:
                 # force to redisplay the hostname
@@ -121,7 +125,8 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
         """
 
         _local_option = option
-        _local_option.state = QtGui.QStyle.State_None
+        if globals.GApp.systconf['general'].draw_selected_rectangle == False:
+            _local_option.state = QtGui.QStyle.State_None
 
         QtSvg.QGraphicsSvgItem.paint(self, painter, _local_option, widget)
 
@@ -200,6 +205,12 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
             event: QtGui.QGraphicsSceneMouseEvent instance
         """
 
+        if globals.workaround_ManualLink == True:
+            # We're not the right recipient of this event,
+            # so acknownledge the workaround and return
+            globals.workaround_ManualLink = False
+            return
+
         if globals.addingLinkFlag and event.button() == QtCore.Qt.LeftButton:
 
             self.__selectedInterface = None
@@ -236,6 +247,8 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
             node: node instance
         """
 
+        # must do this to avoid a bug on Linux (kernel >= 2.6.26)
+        globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
         menu = QtGui.QMenu()
         interfaces_list = self.getInterfaces()
         if len(interfaces_list) == 0:
